//=============================================================================
//  Keke_ChangeLayoutLight - お手軽レイアウト変更
// バージョン: 1.5
//=============================================================================
// Copyright (c) 2020 ケケー
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
//=============================================================================

/*:
 * @target MZ
 * @plugindesc 各種レイアウトをパッと変更する
 * @author ケケー
 * 
 * @help
 * 各種レイアウトをお手軽に変更するよ
 * またオプションでプレイヤーにレイアウトを変更させることもできるよ
 * 
 * ◉ 変更できるレイアウト項目 ◉
 *
 * ▼ メニューコマンド位置
 * メニューコマンド(アイテムとか)を左に置くか右に置くか
 *
 * ▼ バトルコマンド位置
 * バトルコマンド(攻撃とか)を左に置くか右に置くか
 *
 * ▼ メニューヘルプ位置
 * メニューヘルプ(アイテムとかの説明)を上に置くか下に置くか
 *
 * ▼ タッチUI位置
 * タッチUI(戻るボタンとか)を上に置くか下に置くか
 *
 *
 * ◉ 連動設定 ◉
 
 * ▼ 装備コマンド連動
 * メニューコマンド位置を左にしたとき、装備コマンドの位置も左にする(デフォは右固定)
 *
 * ▼ プロフィール連動
 * メニューヘルプ位置を上にしたとき、ステータス画面のプロフィール欄の位置も上にする(デフォは下固定)
 * 
 *
 * ◉ オプション追加 ◉
 * レイアウト設定をオプションに追加し、
 * プレイヤーの好みでレイアウトを変更させることができる
 *
 * ▼ 何をオプションに表示するかや、表示する順番を変えたい
 * パラメータの『オプション追加リスト』で設定する
 * リストから削除すれば表示しなくなり、
 * リストの順番を変えれば表示される順番も変わる
 *
 * ▼ 表示リストを最初の状態に戻したい
 * これをリストのテキスト欄にコピペする
 *  ["メニューコマンド位置","バトルコマンド位置","メニューヘルプ位置","タッチUI位置"]
 *
 * ▼ 実際にオプション上で表示される項目名を変えたい
 * 『-名』のパラメータを変更する
 *
 * 
 * ◉ 利用規約 ◉
 *  MITライセンスのもと、好きに使ってくれて大丈夫
 * ただし作者は何も責任を負わないよ
 * また著作権は『ケケー』にあるよ
 *
 *
 * @param ◉ レイアウト設定 ◉
 *
 * @param メニューコマンド位置
 * @desc メニューコマンドの位置(左か右か)
 * @type select
 * @option 左
 * @option 右
 * @default 右
 *
 * @param バトルコマンド位置
 * @param バトルコマンド位置
 * @desc バトルコマンドの位置(左か右か)
 * @type select
 * @option 左
 * @option 右
 * @default 右
 *
 * @param メニューヘルプ位置
 * @desc メニューヘルプ(スキルとか説明するの)の位置(上か下か)
 * @type select
 * @option 上
 * @option 下
 * @default 下
 *
 * @param タッチUI位置
 * @desc タッチUI(戻るとかのボタン)の位置(上か下か)
 * @type select
 * @option 上
 * @option 下
 * @default 上
 *
 * @param ◉ 連動設定 ◉
 *
 * @param 装備コマンド連動
 * @desc 装備画面のコマンドをメニューコマンド位置に連動させる(デフォは右固定)
 * @type boolean
 * @default true
 *
 * @param プロフィール連動
 * @desc ステータス画面のプロフィールウインドウをヘルプ位置に連動させる(デフォは下固定)
 * @type boolean
 * @default true
 *
 * @param ◉ オプション追加 ◉
 *
 * @param オプション追加リスト
 * @desc オプションに追加する項目。並べ替えると表示順が変わる。追加しない場合はリストから消す
 * @type string[]
 * @default ["メニューコマンド位置","バトルコマンド位置","メニューヘルプ位置","タッチUI位置"]
 * 
 * @param メニューコマンド位置-名
 * @desc メニューコマンド位置のオブション上で表示される名前
 * @type string
 * @default メニューコマンド位置
 *
 * @param バトルコマンド位置-名
 * @desc バトルコマンド位置のオブション上で表示される名前
 * @type string
 * @default バトルコマンド位置
 *
 * @param メニューヘルプ位置-名
 * @desc メニューヘルプ位置のオブション上で表示される名前
 * @type string
 * @default メニューヘルプ位置
 *
 * @param タッチUI位置-名
 * @desc タッチUI位置のオブション上で表示される名前
 * @type string
 * @default タッチUI位置
 */

(() => {
    // プラグイン名
    const pluginName = "Keke_ChangeLayoutLight";
    
    
    //--  パラメータ受け取り  --//
    
    var parameters = PluginManager.parameters(pluginName);
    // レイアウト設定
    const keke_menuCmdPos = parameters["メニューコマンド位置"];
    const keke_battleCmdPos = parameters["バトルコマンド位置"]; 
    const keke_menuHelpPos = parameters["メニューヘルプ位置"];
    const keke_touchUIPos = parameters["タッチUI位置"];
    // 連動設定
    const keke_equipCmdAdapt = eval(parameters["装備コマンド連動"]);
    const keke_profileAdapt = eval(parameters["プロフィール連動"]);
    // オプション追加
    const keke_addOptionItems = eval(parameters["オプション追加リスト"]);
    const keke_menuCmdPosName = parameters["メニューコマンド位置-名"];
    const keke_battleCmdPosName = parameters["バトルコマンド位置-名"];
    const keke_menuHelpPosName = parameters["メニューヘルプ位置-名"];
    const keke_touchUIPosName = parameters["タッチUI位置-名"];
     
     
     
    
    //--  プラグインデータ  --//
    
    // ゲームシステム
    const _Game_System_initialize = Game_System.prototype.initialize;
    Game_System.prototype.initialize = function() {
        _Game_System_initialize.call(this);
        // メニューコマンド位置
        this._menuCmdPosKe = keke_menuCmdPos;
        // バトルコマンド位置
        this._battleCmdPosKe = keke_battleCmdPos;
        // メニューヘルプ位置
        this._menuHelpPosKe = keke_menuHelpPos;
        // タッチUI位置
        this._touchUIPosKe = keke_touchUIPos;
    };
    
    
    
    
    //--  レイアウト設定  --//
    
    // メニューコマンド位置
    const _Scene_MenuBase_isRightInputMode = Scene_MenuBase.prototype.isRightInputMode;
    Scene_MenuBase.prototype.isRightInputMode = function() {
        let result = _Scene_MenuBase_isRightInputMode.call(this);
        switch ($gameSystem._menuCmdPosKe) {
            case "左":
                result = false;
                break;
            case "右":
                result = true;
        }
        return result;
    };
    
    
    // バトルコマンド位置
    const _Scene_Battle_isRightInputMode = Scene_Battle.prototype.isRightInputMode;
    Scene_Battle.prototype.isRightInputMode = function() {
        let result = _Scene_Battle_isRightInputMode.call(this);
        switch ($gameSystem._battleCmdPosKe) {
            case "左":
                result = false;
                break;
            case "右":
                result = true;
        }
        return result;
    };
    
    
    // メニューヘルプ位置
    const _Scene_Base_isBottomHelpMode = Scene_Base.prototype.isBottomHelpMode;
    Scene_Base.prototype.isBottomHelpMode = function() {
        let result = _Scene_Base_isBottomHelpMode.call(this);
        switch ($gameSystem._menuHelpPosKe) {
            case "上":
                result = false;
                break;
            case "下":
                result = true;
        }
        return result;
    };
    
    
    // タッチUI位置
    const _Scene_Base_isBottomButtonMode = Scene_Base.prototype.isBottomButtonMode;
    Scene_Base.prototype.isBottomButtonMode = function() {
        let result = _Scene_Base_isBottomButtonMode.call(this);
        switch ($gameSystem._touchUIPosKe) {
            case "上":
                result = false;
                break;
            case "下":
                result = true;
        }
        return result;
    };
    
    
    
    
    //--  連動設定  --//
    
    // 装備コマンド連動
    const _Scene_Equip_createItemWindow = Scene_Equip.prototype.createItemWindow;
    Scene_Equip.prototype.createItemWindow = function() {
         _Scene_Equip_createItemWindow.call(this);
         if (keke_equipCmdAdapt && !this.isRightInputMode()) {
             this._commandWindow.x -= this._statusWindow.width;
             this._slotWindow.x -= this._statusWindow.width;
             this._itemWindow.x -= this._statusWindow.width;
             this._statusWindow.x += this._commandWindow.width;
          }
    };
    
    
    // プロフィール連動
    const _Scene_Status_createStatusEquipWindow = Scene_Status.prototype.createStatusEquipWindow;
    Scene_Status.prototype.createStatusEquipWindow = function() {
        _Scene_Status_createStatusEquipWindow.call(this);
        if (keke_profileAdapt && !this.isBottomHelpMode()) {
            this._profileWindow.y = this.isBottomButtonMode() ? 0 : this.buttonAreaHeight();
            this._statusWindow.y += this._profileWindow.height;
            this._statusParamsWindow.y += this._profileWindow.height;
            this._statusEquipWindow.y += this._profileWindow.height;
        }
    };
    
    
    
    
    //--  オプション追加  --//
    
    //--  コンフィグマネージャー
    
    // メニューコマンド位置
    Object.defineProperty(ConfigManager, "メニューコマンド位置", {
        get: function() {
            return $gameSystem._menuCmdPosKe;
        },
        set: function(value) {
            $gameSystem._menuCmdPosKe = value;
        },
        configurable: true
    });
    
    // バトルコマンド位置
    Object.defineProperty(ConfigManager, "バトルコマンド位置", {
        get: function() {
            return $gameSystem._battleCmdPosKe;
        },
        set: function(value) {
            $gameSystem._battleCmdPosKe = value;
        },
        configurable: true
    });
    
    // メニューヘルプ位置
    Object.defineProperty(ConfigManager, "メニューヘルプ位置", {
        get: function() {
            return $gameSystem._menuHelpPosKe;
        },
        set: function(value) {
            $gameSystem._menuHelpPosKe = value;
        },
        configurable: true
    });
    
    // タッチUI位置
    Object.defineProperty(ConfigManager, "タッチUI位置", {
        get: function() {
            return $gameSystem._touchUIPosKe;
        },
        set: function(value) {
            $gameSystem._touchUIPosKe = value;
        },
        configurable: true
    });
    
    
    // オプションに項目追加
    const _Window_Options_makeCommandList = Window_Options.prototype.makeCommandList;
    Window_Options.prototype.makeCommandList = function() {
        _Window_Options_makeCommandList.call(this);
        // 追加したフラグ
        let added = false;
        // 追加項目を展開
        for (let item of keke_addOptionItems) {
            // 項目名設定
            let name = "";
            if (item == "メニューコマンド位置") { name = keke_menuCmdPosName; }
            if (item == "バトルコマンド位置") { name = keke_battleCmdPosName; }
            if (item == "メニューヘルプ位置") { name = keke_menuHelpPosName; }
            if (item == "タッチUI位置") { name = keke_touchUIPosName; }
            // 項目追加
            this.addCommand(name, item);
            added = true;
        }
        // 追加したときは
        if (added) {
            // ストリングス追加
            this._posLRStrsKe = ["左", "右"];
            this._posUDStrsKe = ["上", "下"];
        }
        // リサイズ
        this.resizeKe();
    };
    
    
    // オプションのリサイズ
    Window_Options.prototype.resizeKe = function(){
        // コマンド数取得
        let cmdNum = this._list.length;
        // ハイト変更(画面ハイトは超えない)
        let height = Math.min(Graphics.boxHeight, this.fittingHeight(cmdNum));
        this.height = height;
        // Y位置変更
        this.y = (Graphics.boxHeight - this.height) / 2;
    };
    
    
    // オプションウインドウのハイト拡大
    const _Scene_Options_optionsWindowRect = Scene_Options.prototype.optionsWindowRect;
    Scene_Options.prototype.optionsWindowRect = function() {
        let result = _Scene_Options_optionsWindowRect.call(this);
        result.height = Graphics.height;
        return result;
    };
    
    
    // ステータステキスト
    const _Window_Options_statusText = Window_Options.prototype.statusText;
    Window_Options.prototype.statusText = function(index) {
        let result = _Window_Options_statusText.call(this, index);
        // シンボルと値を取得
        const symbol = this.commandSymbol(index);
        const value = this.getConfigValue(symbol);
        // 位置設定のとき
        if (symbol == "メニューコマンド位置" || symbol == "バトルコマンド位置" || symbol == "メニューヘルプ位置" || symbol == "タッチUI位置") {
            return this.stringStatusText(value);
        }
        return result;
    };
    
    
    // 文字列テキストステータス
    Window_Options.prototype.stringStatusText = function(value) {
        return value;
    };
    
    
    // 決定ボタン時の処理
    const _Window_Options_processOk = Window_Options.prototype.processOk;
    Window_Options.prototype.processOk = function() {
        const symbol = this.commandSymbol(this.index());
        // コマンド位置のとき
        if (symbol == "メニューコマンド位置" || symbol == "バトルコマンド位置"){
            this.changeStringKe(symbol, this._posLRStrsKe, true, true);
        // UI位置のとき
        } else if (symbol == "メニューヘルプ位置" || symbol == "タッチUI位置") {
            this.changeStringKe(symbol, this._posUDStrsKe, true, true);
        // それ以外のとき
        } else {
            _Window_Options_processOk.call(this);
        }
    };
    
    
    // 右カーソルの処理
    const _Window_Options_cursorRight = Window_Options.prototype.cursorRight;
    Window_Options.prototype.cursorRight = function() {
        const symbol = this.commandSymbol(this.index());
        // コマンド位置のとき
        if (symbol == "メニューコマンド位置" || symbol == "バトルコマンド位置"){
            this.changeStringKe(symbol, this._posLRStrsKe, true, true);
        // UI位置のとき
        } else if (symbol == "メニューヘルプ位置" || symbol == "タッチUI位置") {
            this.changeStringKe(symbol, this._posUDStrsKe, true, true);
        // それ以外のとき
        } else {
            _Window_Options_cursorRight.call(this);
        }
    };
    
    
    // 左カーソルの処理
    const _Window_Options_cursorLeft = Window_Options.prototype.cursorLeft;
    Window_Options.prototype.cursorLeft = function() {
        const symbol = this.commandSymbol(this.index());
        // コマンド位置のとき
        if (symbol == "メニューコマンド位置" || symbol == "バトルコマンド位置"){
            this.changeStringKe(symbol, this._posLRStrsKe, false, true);
        // UI位置のとき
        } else if (symbol == "メニューヘルプ位置" || symbol == "タッチUI位置") {
            this.changeStringKe(symbol, this._posUDStrsKe, true, true);
        // それ以外のとき
        } else {
            _Window_Options_cursorLeft.call(this);
        }
    };
    
    
    // 文字列の変更
    Window_Options.prototype.changeStringKe = function(symbol, strs, forward, wrap) {
        // ストルスがなかったらリターン
        if (!strs) { return; }
        // テキスト取得
        const text = this.getConfigValue(symbol);
        // インデックス取得
        let index = strs.indexOf(text);
        // インデックス最大数取得
        const iMax = strs.length - 1;
        // インデックスがなかったらリターン
        if (index < 0) { return; }
        // インデックスを変更
        index = index+ (forward ? 1 : -1);
        // ループさせる
        if (wrap) {
            if (index > iMax) { index = 0; }
            if (index < 0) { index = iMax; }
        }
        // 範囲内に収める
        index = index.clamp(0, iMax);
        // 新しい値を取得
        const value = strs[index];
        // セット
        this.setConfigValue(symbol, value);
        this.redrawItem(this.findSymbol(symbol));
        this.playCursorSound();
    };

    
    // コンフィグデータ保存
    const _ConfigManager_makeData = ConfigManager.makeData;
    ConfigManager.makeData = function() {
        let config = _ConfigManager_makeData.call(this);
        // ゲーム開始時用
        config._menuCmdPosKe = $gameSystem._menuCmdPosKe;
        config._battleCmdPosKe = $gameSystem._battleCmdPosKe;
        config._menuHelpPosKe = $gameSystem._menuHelpPosKe;
        config._touchUIPosKe = $gameSystem._touchUIPosKe;
        // ニューゲーム用
        this._menuCmdPosKe = $gameSystem._menuCmdPosKe;
        this._battleCmdPosKe = $gameSystem._battleCmdPosKe;
        this._menuHelpPosKe = $gameSystem._menuHelpPosKe;
        this._touchUIPosKe = $gameSystem._touchUIPosKe;
        return config;
    };
    
    
    // ゲーム開始時に呼び出し
    const _ConfigManager_applyData = ConfigManager.applyData;
    ConfigManager.applyData = function(config) {
        _ConfigManager_applyData.call(this, config);
        this._menuCmdPosKe = config._menuCmdPosKe;
        this._battleCmdPosKe = config._battleCmdPosKe;
        this._menuHelpPosKe = config._menuHelpPosKe;
        this._touchUIPosKe = config._touchUIPosKe;
    };
    
    
    // ニューゲーム時に呼び出し
    const _DataManager_setupNewGame = DataManager.setupNewGame;
    DataManager.setupNewGame = function() {
        _DataManager_setupNewGame.call(this);
        cm = ConfigManager;
        if (cm._menuCmdPosKe) {
            $gameSystem._menuCmdPosKe = cm._menuCmdPosKe;
        }
        if (cm._battleCmdPosKe) {
            $gameSystem._battleCmdPosKe = cm._battleCmdPosKe;
        }
        if (cm._menuHelpPosKe) {
            $gameSystem._menuHelpPosKe = cm._menuHelpPosKe;
        }
        if (cm._touchUIPosKe) {
            $gameSystem._touchUIPosKe = cm._touchUIPosKe;
        }
    };
    
})();